package com.example.api

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.example.api.core.domain.ApiRepository
import com.example.api.core.domain.AuthUserResponse
import com.example.api.core.domain.RegisterUser
import com.example.api.core.domain.Projects
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import javax.inject.Inject

/**
 * Автор: Манякин Дмитрий (user5)
 * Дата создания: 27.05.2025
 * */

@HiltViewModel
class MainViewModel @Inject constructor(
    private val apiRepository: ApiRepository
) : ViewModel() {

    private val _authUiState =
        MutableStateFlow<FetchResultUiState<AuthUserResponse>>(FetchResultUiState.Initial())
    val authUiState: StateFlow<FetchResultUiState<AuthUserResponse>>
        get() = _authUiState.asStateFlow()

    private val _ProjectsUiState =
        MutableStateFlow<FetchResultUiState<Projects>>(FetchResultUiState.Initial())
    val projectsUiState: StateFlow<FetchResultUiState<Projects>>
        get() = _ProjectsUiState.asStateFlow()

    fun auth(email: String, password: String) {
        viewModelScope.launch(Dispatchers.IO) {
            val result = apiRepository.auth(
                RegisterUser(
                    email = email,
                    password = password,
                    passwordConfirm = password
                )
            )

            withContext(Dispatchers.Main) {
                _authUiState.value = result.map(FetchResultMapper())
            }
        }
    }

    fun salesAndProjects() {
        viewModelScope.launch(Dispatchers.IO) {
            val result = apiRepository.projects()

            withContext(Dispatchers.Main) {
                _ProjectsUiState.value = result.map(FetchResultMapper())
            }
        }
    }
}